@@ -29,6 +29,12 @@
 #include <Rcpp/r_cast.h>
 #include <Rcpp/Dimension.h>
 
+namespace Rcpp{
+    namespace traits{
+         template <int RTYPE, bool NA, typename VECTOR> struct Extractor ;  
+    }
+}
+
 namespace Rcpp{
 
 #include <Rcpp/vector/00_forward_Vector.h>
