@@ -2,7 +2,7 @@
 //
 // string_proxy.h: Rcpp R/C++ interface class library -- 
 //
-// Copyright (C) 2010	Dirk Eddelbuettel and Romain Francois
+// Copyright (C) 2010 - 2011 Dirk Eddelbuettel and Romain Francois
 //
 // This file is part of Rcpp.
 //
@@ -265,9 +265,9 @@ namespace internal{
 				return ::Rcpp::as<U>(get()) ;
 			}
 			
-			operator bool() const {
-			    return ::Rcpp::as<bool>(get()) ;
-			}
+			// helping the compiler (not sure why it can't help itself)
+			operator bool() const { return ::Rcpp::as<bool>(get()) ; }
+			operator int() const { return ::Rcpp::as<int>(get()) ; }
 			
 			void swap(generic_proxy& other){
 				SEXP tmp = PROTECT(get()) ;
